from .rpnlib import *

class Turmit(object):
    ''' @brief Represent a turmit that act given an RPN expression with an
        infinite looping stack with variable stack size.
    '''
    
    def __init__(self, stack_size=8):
        ## @brief List that represent a stack
        self._stack = [ 0 for _ in range(stack_size)]
        ## @brief Stack head index
        self._cur = stack_size - 1
        ## @brief Stores turmit direction
        self._dir = 0

    @property
    def shead(self):
        return self._stack[self._cur]

    def _pop(self):
        ''' @brief Pops a value from the stack
            
            @note If stack head is 0, set the stack head to len(self._stack) 
            - 1

            @return popped value
        '''
        res = self._stack[self._cur]
        if self._cur == 0:
            self._cur = len(self._stack) - 1
        else:
            self._cur -= 1
        return res

    def _push(self, val):
        ''' @brief Pushes a value on the stack
            
            @note If no space left on the stack, the head is set to 0
        
            @param val int : value to push
        '''
        self._cur += 1
        if self._cur >= len(self._stack):
            self._cur = 0
        self._stack[self._cur] = val

    @RpnOp
    def mem_sz(self, new_sz):
        if new_sz < 2:
            raise ValueError('New size sould be >= 2')
        stksz = len(self._stack)
        if new_sz > stksz:
            self._stack += [ 0 for _ in range(new_sz - stksz) ]
        elif new_sz < stksz:
            self._stack = self._stack[0:new_sz]
            if self._cur >= new_sz:
                self._cur = 0
    
    @RpnOp
    def add(self, a, b):
        ''' @brief Adds one value to another
            @param a int : value
            @param b int : value to add to a
            @return a + b
        '''
        return a + b

    @RpnOp
    def sub(self, a, b):
        ''' @brief Substitutes a value to another
            @param a int : value
            @param b int : value to substitute to a
            @return a - b
        '''
        return a - b

    @RpnOp
    def bin_and(self, a, b):
        ''' @brief Binary and
             @param a int : value
             @param b int : value
             @return a & b
        '''
        return a & b

    @RpnOp
    def dup(self, a):
        ''' @brief Duplicates stack head
            @param a int : value
            @return a
        '''
        self._push(a)
        return a

    @RpnOp
    def lshift(self, a, b):
        ''' @brief Left shifts a of b
            @param a int : value
            @param b int : value
            @return a << b
        ''' 
        return a << b

    @RpnOp
    def mod(self, a, b):
        ''' @brief Gives a modulo b
            @param a int : value
            @param b int : value
            @return a % b
        '''
        return a % b

    
    @RpnOp
    def mul(self, a, b):
        ''' @brief Multiplies a with b
            @param a int : value
            @param b int : value
            @return a * b
        '''
        return a * b

    @RpnOp
    def bin_or(self, a, b):
        ''' @brief Binary or
            @param a int : value
            @param b int : value
            @return a | b
        '''
        return a | b

    @RpnOp
    def pop(self, a):
        ''' @brief Pops a, head of the stack
            @param a int : value
        '''
        pass
